#!/usr/bin/perl
use MyApp::Schema;

BEGIN {
  $ENV{'DBIC_TRACE'} => 1;
}

use Test::More tests => 2;
use Test::Exception;
use Data::Dumper;

my $schema
    = MyApp::Schema->connect(
             'dbi:mysql:database=dbixTest;host=localhost;mysql_enable_utf8=1',
             'foo', 'bar' );
$schema->storage->debug(1);

createData($schema);

my $artist_rs = $schema->resultset('Artist');

subtest "Test without alias name in +as" => sub {
  plan tests => 5;
  my $spec = { '+as'      => [ 'name',    'number' ],
               '+select'  => [ 'me.name', 'count(the_wall_cds.cdid)' ],
               'columns'  => [],
               'join'     => 'the_wall_cds',
               'group_by' => ['me.name'], };
  my $rs = $artist_rs->search_rs( {}, $spec );

  eval {
    my $query = $rs->as_query;
    my $sql   = shift @$$query;
    note "Sql: $sql, binds " . join( ',', map { $_->[1] } @$$query );
  };

  my @data = eval { $rs->all(); };

  my %data;
  foreach my $row (@data) {
    $DB::single =1;
    my %row_hash = $row->get_columns;
    is (scalar(keys %row_hash), 2, 'should have two columns');
    lives_ok(
      sub {
        $data{ $row->get_column('name') }
            = $row->get_column('number');
      },
      'Data row OK' );
  }
  is_deeply( \%data, { Marillion => 0, 'Pink Floyd' => 1 } );
};
subtest "Test with alias name in +as" => sub {
  plan tests => 5;
  my $spec = { '+as'      => [ 'name',    'the_wall_cds.number' ],
               '+select'  => [ 'me.name', 'count(the_wall_cds.cdid)' ],
               'columns'  => [],
               'join'     => 'the_wall_cds',
               'group_by' => ['me.name'], };
  my $rs = $artist_rs->search_rs( {}, $spec );

  eval {
    my $query = $rs->as_query;
    my $sql   = shift @$$query;
    note "Sql: $sql, binds " . join( ',', map { $_->[1] } @$$query );
  };

  my @data = eval { $rs->all(); };

  my %data;
  foreach my $row (@data) {
    my %row_hash = $row->get_columns;
    is (scalar(keys %row_hash), 2, 'should have two columns');
    lives_ok(
      sub {
        $data{ $row->get_column('name') }
            = $row->get_column('the_wall_cds.number');
      },
      'Data row OK' );
  }
  is_deeply( \%data, { Marillion => 0, 'Pink Floyd' => 1 } );
};

sub createData {
  my $artist_rs = $schema->resultset('Artist');

  my @rows = $artist_rs->all();
  return if @rows;

  my %data = (
     'Pink Floyd' => [ { title => 'A Momentary Lapse of Reason',
                         year  => '1987-01-01'
                       },
                       { title => 'The Wall',          year => '1979-01-01' },
                       { title => 'The Division Bell', year => '1994-01-01' },
     ],
     'Marillion' => [ { title => 'Script for a Jester\'s Tear',
                        year  => '1983-01-01'
                      },
                      { title => 'Fugazi', year => '1984-01-01' },
     ] );
  foreach my $artist ( keys %data ) {
    my $artist_obj = $artist_rs->create( { name => $artist } );
    my $cds = $data{$artist};
    foreach my $cd (@$cds) {
      $artist_obj->add_to_cds($cd);
    }
  }
}
